'' =================================================================================================
''
''   File....... jm_stepper_demo.spin
''   Purpose.... Demonstrates simple control of a unipolar or bipolar stepper motor
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2017 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 19 JUL 2017
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x                                     
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us

  TRM_BAUD = 115_200                                            ' terminal baud rate
  

con { io pins }                                                  
                                                                 
  RX1   = 31  { I }                                             ' serial / programming
  TX1   = 30  { O }                                            
                                                             
  SDA1  = 29  { I/O }                                           ' i2c / eeprom
  SCL1  = 28  { I/O } 

  EPIN  =  4  { O }                                             ' enable (L293D) { optional }
  BnPIN =  3  { O }                                             ' to coils
  AnPIN =  2  { O }
  BPIN  =  1  { O }
  APIN  =  0  { O }
  

con

  #true, ON, OFF                                                ' digital control

  
  N_STEPS = 360_0 / 7_5                                         ' # of full steps in 360 degrees

  MODE = stepper#M_HALF                                         ' run in half step mode
  

obj

' main                                                          ' * master Spin cog
  time    : "jm_time"                                           '   timing and delays
  io      : "jm_io_basic"                                       '   essential io
  stepper : "jm_stepper"                                        ' * non-blockin stepper control
  term    : "jm_fullduplexserial"                               ' * serial IO for terminal

' * uses cog when loaded                                         
                                                                 
                                                                 
var
                                                           
                                                                 
pub main                                                       
                                                                 
  setup

  ' open pst, set serial port, enable terminal, press a key

  term.rxflush
  term.rx
  term.tx(term#CLS)

  repeat
    term.tx(term#CLS)
    time.pause(250)
    
    term.str(string("Stepper Motor Demo", 13))
    term.str(string("Steps/Rev = "))
    term.dec(stepper.steps_per_rev)
    term.txn(13, 2)

    term.str(string("1 revolution @ 5ms/step", 13))
    stepper.steps(stepper.steps_per_rev, 5_0)                             ' time in 0.1ms units
    finish_move
    
    term.str(string("1 revoltution in 1s", 13))
    stepper.timed_steps(stepper.steps_per_rev, 1000_0) 
    finish_move

    repeat 2
      term.str(string("+180 degrees in 1/8s", 13))  
      stepper.timed_degrees(180, 125_0)
      finish_move
      term.str(string("-180 degrees in 1/8s", 13))  
      stepper.timed_degrees(-180, 125_0)
      finish_move
       
    time.pause(1000)


pub finish_move

  repeat while (stepper.running) 
  time.pause(500)           

                                               
pub setup                                                        
                                                                 
'' Setup IO and objects for application                          
                                                                 
  time.start                                                    ' setup timing & delays

  io.start(0, 0)                                                ' clear all pins (master cog)

  stepper.start(N_STEPS, APIN, EPIN, MODE)                      ' start stepper control  
                                                                 
  term.start(RX1, TX1, %0000, 115_200)                          ' start serial for terminal
                                                                 
                       
dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}      